@@ -18,6 +18,18 @@
 #include <linux/time.h>
 #include <linux/module.h>
 #include <linux/videodev2.h>
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 29)
+# include <media/v4l2-device.h>
+#else
+/* dummy v4l2_device struct/functions */
+# define V4L2_DEVICE_NAME_SIZE (20 + 16)
+struct v4l2_device {
+  char name[V4L2_DEVICE_NAME_SIZE];
+};
+static inline int  v4l2_device_register  (void *dev, void *v4l2_dev) { return 0; }
+static inline void v4l2_device_unregister(struct v4l2_device *v4l2_dev) { return; }
+#endif
 #include <media/v4l2-ioctl.h>
 #include <media/v4l2-common.h>
 
@@ -40,7 +52,7 @@ void *v4l2l_vzalloc(unsigned long size)
 #include <linux/sched.h>
 #include <linux/slab.h>
 
-#define V4L2LOOPBACK_VERSION_CODE KERNEL_VERSION(0, 7, 0)
+#define V4L2LOOPBACK_VERSION_CODE KERNEL_VERSION(0, 8, 0)
 
 
 MODULE_DESCRIPTION("V4L2 loopback video device");
@@ -109,6 +121,10 @@ static int video_nr[MAX_DEVICES] = { [0 ... (MAX_DEVICES - 1)] = -1 };
 module_param_array(video_nr, int, NULL, 0444);
 MODULE_PARM_DESC(video_nr, "video device numbers (-1=auto, 0=/dev/video0, etc.)");
 
+static char *card_label[MAX_DEVICES];
+module_param_array(card_label, charp, NULL, 0000);
+MODULE_PARM_DESC(card_label, "card labels for every device");
+
 static bool exclusive_caps[MAX_DEVICES] = { [0 ... (MAX_DEVICES - 1)] = 1 };
 module_param_array(exclusive_caps, bool, NULL, 0444);
 /* FIXXME: wording */
@@ -155,6 +171,7 @@ struct v4l2l_buffer {
 };
 
 struct v4l2_loopback_device {
+	struct v4l2_device v4l2_dev;
 	struct video_device *vdev;
 	/* pixel and stream format */
 	struct v4l2_pix_format pix_format;
@@ -589,7 +606,13 @@ static int vidioc_querycap(struct file *file, void *priv, struct v4l2_capability
 	int devnr = ((struct v4l2loopback_private *)video_get_drvdata(dev->vdev))->devicenr;
 
 	strlcpy(cap->driver, "v4l2 loopback", sizeof(cap->driver));
-	snprintf(cap->card, sizeof(cap->card), "Dummy video device (0x%04X)", devnr);
+
+	if (card_label[devnr] != NULL) {
+		snprintf(cap->card, sizeof(cap->card), card_label[devnr]);
+	} else {
+		snprintf(cap->card, sizeof(cap->card), "Dummy video device (0x%04X)", devnr);
+	}
+
 	snprintf(cap->bus_info, sizeof(cap->bus_info), "v4l2loopback:%d", devnr);
 
 	cap->version = V4L2LOOPBACK_VERSION_CODE;
@@ -841,23 +864,6 @@ static int vidioc_g_fmt_out(struct file *file, void *priv, struct v4l2_format *f
 	 * or whether we have to always provide a valid format
 	 */
 
-	if (0 == dev->ready_for_capture) {
-		/* we are not fixated yet, so return a default format */
-		const struct v4l2l_format *defaultfmt = &formats[0];
-
-		dev->pix_format.width = 0; /* V4L2LOOPBACK_SIZE_DEFAULT_WIDTH; */
-		dev->pix_format.height = 0; /* V4L2LOOPBACK_SIZE_DEFAULT_HEIGHT; */
-		dev->pix_format.pixelformat = defaultfmt->fourcc;
-		dev->pix_format.colorspace = V4L2_COLORSPACE_SRGB; /* do we need to set this ? */
-		dev->pix_format.field = V4L2_FIELD_NONE;
-
-		pix_format_set_size(&fmt->fmt.pix, defaultfmt,
-				dev->pix_format.width, dev->pix_format.height);
-
-		dev->buffer_size = PAGE_ALIGN(dev->pix_format.sizeimage);
-		dprintk("buffer_size = %ld (=%d)\n", dev->buffer_size, dev->pix_format.sizeimage);
-		allocate_buffers(dev);
-	}
 	fmt->fmt.pix = dev->pix_format;
 	return 0;
 }
@@ -1540,6 +1546,7 @@ static int vidioc_dqbuf(struct file *file, void *private_data, struct v4l2_buffe
 		dprintkrw("output DQBUF index: %d\n", b->buffer.index);
 		unset_flags(b);
 		*buf = b->buffer;
+		buf->type = V4L2_BUF_TYPE_VIDEO_OUTPUT;
 		return 0;
 	default:
 		return -EINVAL;
@@ -2059,19 +2066,30 @@ static void timeout_timer_clb(unsigned long nr)
 /* init loopback main structure */
 static int v4l2_loopback_init(struct v4l2_loopback_device *dev, int nr)
 {
+	int ret;
+	snprintf(dev->v4l2_dev.name, sizeof(dev->v4l2_dev.name),
+                        "v4l2loopback-%03d", nr);
+        ret = v4l2_device_register(NULL, &dev->v4l2_dev);
+        if (ret)
+                return ret;
+
 	MARK();
 	dev->vdev = video_device_alloc();
-	if (dev->vdev == NULL)
+	if (dev->vdev == NULL) {
+		v4l2_device_unregister(&dev->v4l2_dev);
 		return -ENOMEM;
+	}
 
 	video_set_drvdata(dev->vdev, kzalloc(sizeof(struct v4l2loopback_private), GFP_KERNEL));
 	if (video_get_drvdata(dev->vdev) == NULL) {
+		v4l2_device_unregister(&dev->v4l2_dev);
 		kfree(dev->vdev);
 		return -ENOMEM;
 	}
 	((struct v4l2loopback_private *)video_get_drvdata(dev->vdev))->devicenr = nr;
 
 	init_vdev(dev->vdev, nr);
+	dev->vdev->v4l2_dev = &dev->v4l2_dev;
 	init_capture_param(&dev->capture_param);
 	set_timeperframe(dev, &dev->capture_param.timeperframe);
 	dev->keep_format = 0;
@@ -2106,6 +2124,17 @@ static int v4l2_loopback_init(struct v4l2_loopback_device *dev, int nr)
 
 	/* FIXME set buffers to 0 */
 
+	/* Set initial format */
+	dev->pix_format.width = 0; /* V4L2LOOPBACK_SIZE_DEFAULT_WIDTH; */
+	dev->pix_format.height = 0; /* V4L2LOOPBACK_SIZE_DEFAULT_HEIGHT; */
+	dev->pix_format.pixelformat = formats[0].fourcc;
+	dev->pix_format.colorspace = V4L2_COLORSPACE_SRGB; /* do we need to set this ? */
+	dev->pix_format.field = V4L2_FIELD_NONE;
+
+	dev->buffer_size = PAGE_ALIGN(dev->pix_format.sizeimage);
+	dprintk("buffer_size = %ld (=%d)\n", dev->buffer_size, dev->pix_format.sizeimage);
+	allocate_buffers(dev);
+
 	init_waitqueue_head(&dev->read_event);
 
 	MARK();
@@ -2200,6 +2229,7 @@ static void free_devices(void)
 			v4l2loopback_remove_sysfs(devs[i]->vdev);
 			kfree(video_get_drvdata(devs[i]->vdev));
 			video_unregister_device(devs[i]->vdev);
+			v4l2_device_unregister(&devs[i]->v4l2_dev);
 			kfree(devs[i]);
 			devs[i] = NULL;
 		}
